import {
  BugReportOutlined,
  FormatPaintOutlined,
  SettingsOutlined,
  WallpaperOutlined,
  WebAssetOutlined,
} from "@mui/icons-material";
import { TabContext, TabList, TabPanel } from "@mui/lab";
import { Box, Button, IconButton, Tab, Tooltip, useTheme } from "@mui/material";
import { useAtom } from "jotai";
import { useEffect, useState } from "react";
import { useSettings } from "../../../contexts/SettingsContext";
import { stagedSettingsAtom } from "../../../lib/store/jotai/settings";
import { FloatingDialog } from "../../Generic/FloatingDialog";
import { Background } from "./SettingsTabs/Background";
import { Debug } from "./SettingsTabs/Debug";
import { Style } from "./SettingsTabs/Style";
import { Window } from "./SettingsTabs/Window";

export const Settings = () => {
  // contexts
  const theme = useTheme();
  const { fetchSettings, settings, updateSettings } = useSettings();

  // atoms
  const [stagedSettings, setStagedSettings] = useAtom(stagedSettingsAtom);

  // states
  const [settingsOpenState, setSettingsOpenState] = useState<boolean>(false);
  const [settingsMaximisedState, setSettingsMaximisedState] = useState<boolean>(false);
  const [subTabValue, setSubTabValue] = useState("style");

  const toggleSettings = () => {
    setSettingsOpenState((prevState) => !prevState);
  };

  const closeSettings = () => {
    setSettingsOpenState(false);
  };

  // logic for switching tabs
  const subTabChangeEvent = (newTabValue: string) => {
    setSubTabValue(newTabValue);
  };

  // set staged settings back to current settings on cancel
  const cancelClickEvent = () => {
    setStagedSettings(settings);
    setSettingsOpenState(false);
  };

  const applyClickEvent = () => {
    updateSettings(stagedSettings);
  };

  const saveClickEvent = () => {
    applyClickEvent();

    closeSettings();
  };

  // set staged settings back to current settings on close
  useEffect(() => {
    if (settingsOpenState) fetchSettings();
    if (!settingsOpenState) setStagedSettings(settings);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [settingsOpenState]);

  return (
    <FloatingDialog
      body={
        <Box
          sx={{
            border: "1px solid " + theme.palette.grey[700],
            borderRadius: settings.style.radius + "px",
            display: "flex",
            flexDirection: "column",
            flexGrow: 1,
            my: 2,
            overflow: "hidden",
            p: 0,
          }}
        >
          <TabContext value={subTabValue}>
            <TabList
              onChange={(e, value) => {
                subTabChangeEvent(value);
              }}
              scrollButtons={true}
              sx={{
                borderBottom: "1px solid " + theme.palette.divider,
              }}
              variant="scrollable"
            >
              <Tab
                icon={<FormatPaintOutlined />}
                label="Style"
                value="style"
              />
              <Tab
                icon={<WallpaperOutlined />}
                label="Background"
                value="background"
              />
              <Tab
                icon={<WebAssetOutlined />}
                label="Window"
                value="window"
              />
              <Tab
                icon={<BugReportOutlined />}
                label="Debug"
                value="debug"
              />
            </TabList>
            <Box
              sx={{
                height: "100%",
                overflowY: "auto",
                m: 0,
                width: "100%",
              }}
            >
              <TabPanel
                sx={{ p: 2 }}
                value="style"
              >
                <Style />
              </TabPanel>
              <TabPanel
                sx={{ p: 2 }}
                value="background"
              >
                <Background />
              </TabPanel>
              <TabPanel
                sx={{ p: 2 }}
                value="window"
              >
                <Window />
              </TabPanel>
              <TabPanel
                sx={{ p: 2 }}
                value="debug"
              >
                <Debug />
              </TabPanel>
            </Box>
          </TabContext>
        </Box>
      }
      bottomBar={
        <Box
          sx={{
            display: "flex",
            justifyContent: "flex-end",
            px: 0,
          }}
        >
          <Button
            onClick={cancelClickEvent}
            size="small"
            sx={{
              mr: 1,
            }}
            variant="outlined"
          >
            Cancel
          </Button>

          <Button
            onClick={applyClickEvent}
            size="small"
            sx={{
              mr: 1,
            }}
            variant="outlined"
          >
            Apply
          </Button>
          <Button
            onClick={saveClickEvent}
            size="small"
            variant="contained"
          >
            Save
          </Button>
        </Box>
      }
      close={closeSettings}
      maximisedState={settingsMaximisedState}
      openButton={
        <Tooltip title="Settings">
          <IconButton
            onClick={toggleSettings}
            size="small"
          >
            <SettingsOutlined />
          </IconButton>
        </Tooltip>
      }
      openState={settingsOpenState}
      setMaximisedState={setSettingsMaximisedState}
      title="Settings"
    />
  );
};
